@@ -24,20 +24,16 @@ memory requirements
 Created: Fri 14 May 2021 05:10:18 PM EDT
 """
 
-import json
+import json, logging, os
 from enum import Enum
 from dataclasses import dataclass
 from .filehelper import open_for_read
-from .constants import BLASTDB_ERROR, INPUT_ERROR
-from .base import DBSource
+from .constants import BLASTDB_ERROR, INPUT_ERROR, ELB_BLASTDB_MEMORY_MARGIN
+from .constants import UNKNOWN_ERROR, MolType
+from .constants import ELB_S3_PREFIX, ELB_GCS_PREFIX
+from .base import DBSource, PositiveInteger, MemoryStr
 from .util import UserReportError, get_query_batch_size
-
-
-class MolType(Enum):
-    """Sequence molecular type"""
-    PROTEIN = 'prot'
-    NUCLEOTIDE = 'nucl'
-
+from .aws_traits import get_instance_type_offerings, get_suitable_instance_types
 
 @dataclass
 class SeqData:
@@ -76,36 +72,37 @@ MAX_MT_ONE_DB_LENGTH_NUCL = 14000000000
 NUM_THREADS_MT_ZERO = 16
 
 
-def get_db_data(db: str, source: DBSource) -> DbData:
+def get_db_data(db: str, dbtype: MolType, source: DBSource) -> DbData:
     """
     Read database metadata.
 
     Arguments:
         db: Database name or URI
+        dbtype: Molecule type for BLASTDB
         source: Source for NCBI provided database, ignored for a user database
     """
-    DB_BUCKET_AWS = 's3://ncbi-blast-databases'
-    DB_BUCKET_GCP = 'gs://blast-db'
+    DB_BUCKET_AWS = os.path.join(ELB_S3_PREFIX, 'ncbi-blast-databases')
+    DB_BUCKET_GCP = os.path.join(ELB_GCS_PREFIX, 'blast-db')
     DB_BUCKET_NCBI = 'ftp://ftp.ncbi.nlm.nih.gov/blast/db'
 
     # for user databases
-    metadata_file = db + '.json'
+    metadata_file = f"{db}-{dbtype}-metadata.json"
 
     # if an NCBI-provided database
-    if not db.startswith('s3://') and not db.startswith('gs://'):
+    if not db.startswith(ELB_S3_PREFIX) and not db.startswith(ELB_GCS_PREFIX):
         if source == DBSource.AWS or source == DBSource.GCP:
             bucket = DB_BUCKET_AWS if source == DBSource.AWS else DB_BUCKET_GCP
             with open_for_read(f'{bucket}/latest-dir') as f:
-                metadata_file = f'{bucket}/{f.readline().rstrip()}/{db}.json'
+                metadata_file = os.path.join(f'{bucket}/{f.readline().rstrip()}', metadata_file)
         else:
-            metadata_file = f'{DB_BUCKET_NCBI}/{db}.json'
-
+            metadata_file = os.path.join(f'{DB_BUCKET_NCBI}', metadata_file)
+    logging.debug(f'BLASTDB metadata file: {metadata_file}')
     try:
         with open_for_read(metadata_file) as f:
             db_metadata = json.load(f)
     except:
-        msg = f'Metadata file for database: "{db}" does not exist'
-        if db.startswith('s3://') or db.startswith('gs://'):
+        msg = f'Metadata file for database: "{db}" does not exist in {source}'
+        if db.startswith(ELB_S3_PREFIX) or db.startswith(ELB_GCS_PREFIX):
             msg += ' or you lack credentials to access this file'
         msg += '.'
         raise UserReportError(returncode=BLASTDB_ERROR, message=msg)
@@ -175,3 +172,72 @@ def get_batch_length(program: str, mt_mode: MTMode, num_cpus: int) -> int:
         batch_len *= num_cpus
 
     return batch_len
+
+
+def get_mem_limit(db: DbData, const_limit: MemoryStr, db_factor: float,
+                  with_optimal: bool) -> MemoryStr:
+    """Get memory limit for searching a single query batch
+
+    Arguments:
+        db: Database information
+        const_limit: A constant memory limit
+        db_factor: If larger than 0, memory limit will be computed as databse
+                   bytes-to-cache times db_factor
+        with_optimal: Set memory limit for optimal instance type in AWS
+
+    Returns:
+        A search job memory limit as MemoryStr"""
+    if db_factor > 0.0:
+
+        return MemoryStr(f'{round(db.bytes_to_cache_gb * db_factor, 1)}G')
+    else:
+        if with_optimal:
+            result = 60 if db.bytes_to_cache_gb >= 60 else db.bytes_to_cache_gb + 2
+            return MemoryStr(f'{result}G')
+        else:
+            return const_limit
+
+
+def get_machine_type(db: DbData, num_cpus: PositiveInteger, region: str) -> str:
+    """Select a machine type that can acomodate the database and and has
+    enough VCPUs. The machine type is selected from a set of machines supported
+    by AWS Batch and offered in a specific region.
+
+    Arguments:
+        db: Database information
+        num_cpus: Required number of CPUs
+        region: Cloud provided region
+
+    Returns:
+        Instance type with at least as much memory as db.bytes_to_cache_gb and
+        required number of CPUs"""
+
+    AWS_BATCH_SUPPORTED_INSTANCE_TYPES = ['m6g.xlarge', 'r5d.24xlarge', 
+          'm3.xlarge', 'r4.16xlarge', 'r5a.2xlarge', 'c6g.4xlarge',
+          'm6gd.xlarge', 'm5.xlarge', 'c5a.2xlarge', 'r6g.4xlarge',
+          'r6g.16xlarge', 'i3.4xlarge', 'z1d.3xlarge', 'm5n.24xlarge',
+          'a1.medium', 'd3en.2xlarge', 'c6gd.12xlarge', 'r5b.16xlarge',
+          'm5.large', 'c5d.large', 'm6g.2xlarge', 'm5dn.2xlarge', 'c5.large',
+          'g4dn.2xlarge', 'c5.metal', 'i3en.6xlarge', 'inf1.2xlarge',
+          'd3.4xlarge', 'r6gd.4xlarge', 'm5.2xlarge', 'r6g.xlarge',
+          'm5dn.8xlarge', 'r5n.16xlarge', 'm6g.8xlarge', 'm6gd.12xlarge',
+          'c5a.8xlarge', 'i2.xlarge', 'm5d.12xlarge', 'm5.metal', 'c5d.metal',
+          'm4.4xlarge', 'm5.12xlarge', 'm6g.12xlarge', 'r5n.4xlarge',
+          'm6gd.4xlarge', 'd3en.8xlarge', 'c4.large', 'c5d.2xlarge',
+          'r5d.2xlarge', 'r5.xlarge', 'r5b.24xlarge', 'c4.4xlarge', 'c4.xlarge',
+          'c6g.large', 'c6gd.medium', 'r6gd.12xlarge', 'r4.8xlarge',
+          'm5d.xlarge', 'c6gd.2xlarge', 'm5.8xlarge', 'c5.2xlarge',
+          'g3.8xlarge', 'c5n.9xlarge']
+
+    # get a list of instance types offered in the region
+    offerings = get_instance_type_offerings(region)
+
+    supported_offerings = [tp for tp in offerings if tp in AWS_BATCH_SUPPORTED_INSTANCE_TYPES]
+
+    # get properties of suitable instances
+    suitable_props = get_suitable_instance_types(min_memory=MemoryStr(f'{db.bytes_to_cache_gb * ELB_BLASTDB_MEMORY_MARGIN}G'),
+                                                 min_cpus=num_cpus,
+                                                 instance_types=supported_offerings)
+
+    return sorted(suitable_props, key=lambda x: x['VCpuInfo']['DefaultVCpus'])[0]['InstanceType']
+
